@@ -13,7 +13,7 @@
 #
 # Credits: (see the CREDITS file bundled with the psad sources.)
 #
-# Copyright (C) 1999-2012 Michael Rash (mbr@cipherdyne.org)
+# Copyright (C) 1999-2017 Michael Rash (mbr@cipherdyne.org)
 #
 # License (GNU Public License):
 #
@@ -56,6 +56,11 @@ my $log_and_drop_table = 'filter';
 my $enable_ipv6 = 0;
 my $psad_lib_dir = '';
 
+my $USE_IPV6 = 1;
+my $NO_IPV6  = 0;
+
+my %ipt_opts = ();
+
 &usage(1) unless (GetOptions(
     'config=s'    => \$config_file, # Specify path to configuration file.
     'fw-file=s'   => \$fw_file,     # Analyze ruleset contained within
@@ -105,6 +110,19 @@ $enable_ipv6 = 1 if $config{'ENABLE_IPV6_DETECTION'} eq 'Y';
 ### import psad perl modules
 &import_psad_perl_modules();
 
+$ipt_opts{'tmpdir'}     = $config{'PSAD_DIR'};
+$ipt_opts{'iptout_pat'} = $config{'IPT_OUTPUT_PATTERN'};
+$ipt_opts{'ipterr_pat'} = $config{'IPT_ERROR_PATTERN'};
+
+if ($config{'ENABLE_OVERRIDE_FW_CMD'} eq 'Y') {
+    if ($config{'FW_CMD_ARGS'} ne 'NONE') {
+        $ipt_opts{'firewall-cmd'} = $config{'FW_CMD'};
+        $ipt_opts{'fwd_args'}     = $config{'FW_CMD_ARGS'};
+    } else {
+        $ipt_opts{'iptables'} = $config{'FW_CMD'};
+    }
+}
+
 open FWCHECK, "> $config{'FW_CHECK_FILE'}" or die "[*] Could not ",
     "open $config{'FW_CHECK_FILE'}: $!";
 
@@ -131,14 +149,14 @@ sub fw_check() {
     my $send_alert = 0;
 
     my $forward_chain_rv = 1;
-    my $input_chain_rv = &ipt_chk_chain('INPUT', $cmds{'iptables'});
+    my $input_chain_rv = &ipt_chk_chain('INPUT', $NO_IPV6);
     unless ($input_chain_rv) {
         &print_fw_help('INPUT', $cmds{'iptables'});
         $send_alert = 1;
     }
 
     if ($enable_ipv6) {
-        my $tmp_rv = &ipt_chk_chain('INPUT', $cmds{'ip6tables'});
+        my $tmp_rv = &ipt_chk_chain('INPUT', $USE_IPV6);
         unless ($tmp_rv) {
             &print_fw_help('INPUT', $cmds{'ip6tables'});
             $send_alert = 1;
@@ -150,7 +168,7 @@ sub fw_check() {
     ### turned on, so we only check the FORWARD iptables chain if we
     ### do and we have multiple interfaces on the box.
     if (&check_forwarding()) {
-        $forward_chain_rv = &ipt_chk_chain('FORWARD', $cmds{'iptables'});
+        $forward_chain_rv = &ipt_chk_chain('FORWARD', $NO_IPV6);
         unless ($forward_chain_rv) {
             &print_fw_help('FORWARD', $cmds{'iptables'});
             $send_alert = 1;
@@ -190,15 +208,16 @@ sub fw_check() {
         print "[+] Results in $config{'FW_CHECK_FILE'}\n",
             "[+] Exiting.\n";
     }
-    return $forward_chain_rv and $input_chain_rv;
+    return 1 if ($forward_chain_rv and $input_chain_rv);
+    return 0;
 }
 
 sub print_fw_help() {
-    my ($chain, $ipt_bin) = @_;
+    my $chain = shift;
     print FWCHECK
-"[-] You may just need to add a default logging rule to the $ipt_bin\n",
-"    '$log_and_drop_table' '$chain' chain on $config{'HOSTNAME'}.  For more information,\n",
-"    see the file \"FW_HELP\" in the psad sources directory or visit:\n\n",
+"[-] You may just need to add a default logging rule to the\n",
+"    '$log_and_drop_table' '$chain' chain on $config{'HOSTNAME'}. For more information,\n",
+"    see the file \"README\" in the psad sources directory or visit:\n\n",
 "    http://www.cipherdyne.org/psad/docs/fwconfig.html\n\n";
     return;
 }
@@ -271,14 +290,18 @@ sub check_forwarding() {
 }
 
 sub ipt_chk_chain() {
-    my ($chain, $ipt_bin) = @_;
+    my ($chain, $use_ipv6) = @_;
     my $rv = 1;
 
-    my $ipt = new IPTables::Parse 'iptables' => $ipt_bin
+    my $ipt;
+
+    $ipt_opts{'use_ipv6'} = $use_ipv6;
+
+    $ipt = IPTables::Parse->new(%ipt_opts)
         or die "[*] Could not acquire IPTables::Parse object: $!";
 
     if ($fw_analyze) {
-        print "[+] Parsing $ipt_bin $chain chain rules.\n";
+        print "[+] Parsing $chain chain rules.\n";
     }
 
     if ($fw_search_all) {
@@ -310,7 +333,7 @@ sub ipt_chk_chain() {
                 return 0;
             } else {
                 print FWCHECK
-"[-] Could not determine whether the $ipt_bin $chain chain is configured with\n",
+"[-] Could not determine whether the $chain chain is configured with\n",
 "    a default logging rule on $config{'HOSTNAME'}.\n\n";
                 return 0;
             }
@@ -346,7 +369,7 @@ sub ipt_chk_chain() {
                         $str2 = "$proto scans";
                     }
                     print FWCHECK
-"[-] The $chain chain in the $ipt_bin ruleset on $config{'HOSTNAME'} does not\n",
+"[-] The $chain chain in the ruleset on $config{'HOSTNAME'} does not\n",
 "    appear to include a default LOG rule $str1.  psad will not be able to\n",
 "    detect $str2 without such a rule.\n\n";
 
@@ -360,12 +383,12 @@ sub ipt_chk_chain() {
                     }
                     unless ($found) {
                         if ($proto eq 'all') {
-                            $str1 = "[-] The $chain chain in the $ipt_bin ruleset " .
+                            $str1 = "[-] The $chain chain in the ruleset " .
                             "on $config{'HOSTNAME'} includes a default\n    LOG rule for " .
                             "all protocols,";
                             $str2 = 'scans';
                         } else {
-                            $str1 = "[-] The $chain chain in the $ipt_bin ruleset " .
+                            $str1 = "[-] The $chain chain in the ruleset " .
                             "on $config{'HOSTNAME'} inclues a default\n    LOG rule for " .
                             "the $proto protocol,";
                             $str2 = "$proto scans";
@@ -386,7 +409,7 @@ sub ipt_chk_chain() {
                         $str1 = "for the $proto protocol";
                     }
                     print FWCHECK
-"[-] The $chain chain in the $ipt_bin ruleset on $config{'HOSTNAME'} does not\n",
+"[-] The $chain chain in the ruleset on $config{'HOSTNAME'} does not\n",
 "    appear to include a default DROP rule $str1.\n\n";
                     $rv = 0;
                 }
@@ -513,6 +536,12 @@ sub import_config() {
         }
     }
 
+    if ($config{'ENABLE_OVERRIDE_FW_CMD'} eq 'Y') {
+        die "[*] Must set a path to a firewall binary with FW_CMD"
+            if $config{'FW_CMD'} eq 'NONE';
+        $cmds{'iptables'} = $config{'FW_CMD'};
+    }
+
     return;
 }
 
