@@ -11,7 +11,7 @@
 #
 # Credits:  (see the CREDITS file)
 #
-# Copyright (C) 1999-2012 Michael Rash (mbr@cipherdyne.org)
+# Copyright (C) 1999-2018 Michael Rash (mbr@cipherdyne.org)
 #
 # License (GNU Public License):
 #
@@ -53,6 +53,7 @@ my $makeCmd      = '/usr/bin/make';
 my $perlCmd      = '/usr/bin/perl';
 my $wgetCmd      = '/usr/bin/wget';
 my $runlevelCmd  = '/sbin/runlevel';
+my $systemctlCmd = '/bin/systemctl';
 
 my $install_root = '/';
 
@@ -98,6 +99,10 @@ my %required_perl_modules = (
         'force-install' => 0,
         'mod-dir' => 'Storable'
     },
+    'Carp::Clan', => {
+        'force-install' => 0,
+        'mod-dir' => 'Carp-Clan'
+    },
     'Date::Calc', => {
         'force-install' => 0,
         'mod-dir' => 'Date-Calc'
@@ -120,6 +125,7 @@ my @ordered_modules = (qw/
     Unix::Syslog
     Bit::Vector
     Storable
+    Carp::Clan
     Date::Calc
     NetAddr::IP
     IPTables::Parse
@@ -172,7 +178,9 @@ my $locale = 'C';  ### default LC_ALL env variable
 my $no_locale = 0;
 my $deps_dir = 'deps';
 my $init_dir = '/etc/init.d';
+my $systemd_init_dir = '/lib/systemd/system';
 my $init_name = 'psad';
+my $systemd_init_name = 'psad.service';
 my $install_syslog_fifo = 0;
 my $runlevel = -1;
 my @installation_lines = ();
@@ -196,6 +204,7 @@ Getopt::Long::Configure('no_ignore_case');
     'no-syslog-test'    => \$skip_syslog_test,
     'uninstall'         => \$uninstall,   ### Uninstall psad.
     'init-dir=s'        => \$init_dir,
+    'systemd-init-dir=s' => \$systemd_init_dir,
     'init-name=s'       => \$init_name,
     'install-syslog-fifo' => \$install_syslog_fifo,
     'install-root=s'    => \$install_root,
@@ -237,25 +246,36 @@ $cmds{'runlevel'} = $runlevelCmd;
 
 my $distro = &get_distro();
 
-if ($distro eq 'redhat' or $distro eq 'fedora') {
-    ### add chkconfig only if we are runing on a redhat distro
-    $cmds{'chkconfig'} = $chkconfigCmd;
-} elsif ($distro eq 'gentoo') {
-    ### add rc-update if we are running on a gentoo distro
-    $cmds{'rc-update'} = $rcupdateCmd;
-} elsif ($distro eq 'ubuntu') {
-    ### add update-rc.d if we are running on an ubuntu distro
-    $cmds{'update-rc.d'} = $updatercdCmd;
-}
+### handle systems with systemd
+my $is_systemd = &look_for_process(qr|/systemd|);
 
-unless (-d $init_dir) {
-    if (-d '/etc/rc.d/init.d') {
-        $init_dir = '/etc/rc.d/init.d';
-    } elsif (-d '/etc/rc.d') {  ### for Slackware
-        $init_dir = '/etc/rc.d';
-    } else {
-        die "[*] Cannot find the init script directory, use ",
-            "--init-dir <path>" unless $install_test_dir;
+if ($is_systemd) {
+    $init_dir  = $systemd_init_dir;
+    $init_name = $systemd_init_name;
+    $cmds{'systemctl'} = $systemctlCmd;
+    die "[*] systemd init directory $init_dir does not exist, ",
+        "use --systemd-init-dir <path>" unless -d $init_dir;
+} else {
+    if ($distro eq 'redhat' or $distro eq 'fedora') {
+        ### add chkconfig only if we are runing on a redhat distro
+        $cmds{'chkconfig'} = $chkconfigCmd;
+    } elsif ($distro eq 'gentoo') {
+        ### add rc-update if we are running on a gentoo distro
+        $cmds{'rc-update'} = $rcupdateCmd;
+    } elsif ($distro eq 'ubuntu') {
+        ### add update-rc.d if we are running on an ubuntu distro
+        $cmds{'update-rc.d'} = $updatercdCmd;
+    }
+
+    unless (-d $init_dir) {
+        if (-d '/etc/rc.d/init.d') {
+            $init_dir = '/etc/rc.d/init.d';
+        } elsif (-d '/etc/rc.d') {  ### for Slackware
+            $init_dir = '/etc/rc.d';
+        } else {
+            die "[*] Cannot find the init script directory, use ",
+                "--init-dir <path>" unless $install_test_dir;
+        }
     }
 }
 
@@ -275,12 +295,11 @@ if ($uninstall) {
     &uninstall();
 } else {
     &install();
+    open F, "> $config{'INSTALL_LOG_FILE'}" or die $!;
+    print F for @installation_lines;
+    close F;
 }
 
-open F, "> $config{'INSTALL_LOG_FILE'}" or die $!;
-print F for @installation_lines;
-close F;
-
 ### restore the original psad.conf file (this is just the local one in the
 ### sources directory).
 if (-e "${psad_conf_file}.orig") {
@@ -341,8 +360,8 @@ sub install() {
     if ($install_syslog_fifo) {
         unless (-e $config{'PSAD_FIFO_FILE'}) {
             &logr("[+] Creating named pipe $config{'PSAD_FIFO_FILE'}\n");
-            unless (((system "$cmds{'mknod'} -m 600 " .
-                    "$config{'PSAD_FIFO_FILE'} p")>>8) == 0) {
+            unless (&run_cmd("$cmds{'mknod'} -m 600 " .
+                    "$config{'PSAD_FIFO_FILE'} p") == 0) {
                 &logr("[*] Could not create the named pipe " .
                     "\"$config{'PSAD_FIFO_FILE'}\"!\n" .
                     "[*] psad requires this file to exist!  Aborting install.\n");
@@ -382,7 +401,7 @@ sub install() {
     }
     if (-d 'deps' and -d 'deps/whois') {
         &logr("[+] Compiling Marco d'Itri's whois client\n");
-        system "$cmds{'make'} -C deps/whois";
+        &run_cmd("$cmds{'make'} -C deps/whois");
         if (-e 'deps/whois/whois') {
             ### if an old whois process is still around ("text file
             ### busy" error), then it is ok to not be able to copy
@@ -440,14 +459,14 @@ sub install() {
     unlink 'psadwatchd' if -e 'psadwatchd';
 
     ### compile the C psad daemons
-    system $cmds{'make'};
+    &run_cmd($cmds{'make'});
     &logr("[-] Could not compile kmsgsd.c.\n") unless (-e 'kmsgsd');
     &logr("[-] Could not compile psadwatchd.c.\n") unless (-e 'psadwatchd');
 
     ### install fwcheck_psad.pl
     print "\n\n";
     &logr("[+] Verifying compilation of fwcheck_psad.pl script:\n");
-    unless (((system "$cmds{'perl'} -c fwcheck_psad.pl")>>8) == 0) {
+    unless (&run_cmd("$cmds{'perl'} -c fwcheck_psad.pl") == 0) {
         die "[*] fwcheck_psad.pl does not compile with \"perl -c\".  Download ",
             "the latest sources from:\n\nhttp://www.cipherdyne.org/"
             unless $skip_module_install;
@@ -456,7 +475,7 @@ sub install() {
     ### make sure the psad (perl) daemon compiles.  The other three
     ### daemons have all been re-written in C.
     &logr("[+] Verifying compilation of psad perl daemon:\n");
-    unless (((system "$cmds{'perl'} -c psad")>>8) == 0) {
+    unless (&run_cmd("$cmds{'perl'} -c psad") == 0) {
         die "[*] psad does not compile with \"perl -c\".  Download the",
             " latest sources from:\n\nhttp://www.cipherdyne.org/"
             unless $skip_module_install;
@@ -464,7 +483,7 @@ sub install() {
 
     ### install nf2csv
     &logr("[+] Verifying compilation of nf2csv script:\n");
-    unless (((system "$cmds{'perl'} -c nf2csv")>>8) == 0) {
+    unless (&run_cmd("$cmds{'perl'} -c nf2csv") == 0) {
         die "[*] nf2csv does not compile with \"perl -c\".  Download ",
             "the latest sources from:\n\nhttp://www.cipherdyne.org/"
             unless $skip_module_install;
@@ -648,7 +667,7 @@ sub install() {
             if ($syslog_str eq 'syslogd') {
                 if (-e $syslog_conf) {
                     &append_fifo_syslog($syslog_conf);
-                    if (((system "$cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
+                    if (&run_cmd("$cmds{'killall'} -HUP syslogd 2> /dev/null") == 0) {
                         &logr("[+] HUP signal sent to syslogd.\n");
                         $restarted_syslog = 1;
                     }
@@ -656,7 +675,7 @@ sub install() {
             } elsif ($syslog_str eq 'rsyslogd') {
                 if (-e $syslog_conf) {
                     &append_fifo_syslog($syslog_conf);
-                    if (((system "$cmds{'killall'} -HUP rsyslogd 2> /dev/null")>>8) == 0) {
+                    if (&run_cmd("$cmds{'killall'} -HUP rsyslogd 2> /dev/null") == 0) {
                         &logr("[+] HUP signal sent to rsyslogd.\n");
                         $restarted_syslog = 1;
                     }
@@ -665,7 +684,7 @@ sub install() {
             } elsif ($syslog_str eq 'syslog-ng') {
                 if (-e $syslog_conf) {
                     &append_fifo_syslog_ng($syslog_conf);
-                    if (((system "$cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
+                    if (&run_cmd("$cmds{'killall'} -HUP syslog-ng 2> /dev/null") == 0) {
                         &logr("[+] HUP signal sent to syslog-ng.\n");
                         $restarted_syslog = 1;
                     }
@@ -698,20 +717,20 @@ sub install() {
                         my $restarted = 0;
                         if ($syslog_str eq 'syslog-ng') {
                             if (-e "$init_dir/syslog-ng") {
-                                system "$init_dir/syslog-ng restart";
+                                &run_cmd("$init_dir/syslog-ng restart");
                                 $restarted = 1;
                             }
                         } elsif ($syslog_str eq 'rsyslogd') {
                             if (-e "$init_dir/sysklogd") {
-                                system "$init_dir/sysklogd restart";
+                                &run_cmd("$init_dir/sysklogd restart");
                                 $restarted = 1;
                             } elsif (-e "$init_dir/syslog") {
-                                system "$init_dir/syslog restart";
+                                &run_cmd("$init_dir/syslog restart");
                                 $restarted = 1;
                             }
                         } else {
                             if (-e "$init_dir/rsyslog") {
-                                system "$init_dir/rsyslog restart";
+                                &run_cmd("$init_dir/rsyslog restart");
                                 $restarted = 1;
                             }
                         }
@@ -746,21 +765,30 @@ sub install() {
 
     my $init_file = '';
     my $installed_init_script = 0;
-    if ($distro eq 'redhat') {
-        $init_file = 'init-scripts/psad-init.redhat';
-    } elsif ($distro eq 'fedora') {
-        $init_file = 'init-scripts/psad-init.fedora';
-    } elsif ($distro eq 'gentoo') {
-        $init_file = 'init-scripts/psad-init.gentoo';
+
+    if ($is_systemd) {
+        $init_file = 'init-scripts/systemd/psad.service';
     } else {
-        $init_file = 'init-scripts/psad-init.generic';
+        if ($distro eq 'redhat') {
+            $init_file = 'init-scripts/psad-init.redhat';
+        } elsif ($distro eq 'fedora') {
+            $init_file = 'init-scripts/psad-init.fedora';
+        } elsif ($distro eq 'gentoo') {
+            $init_file = 'init-scripts/psad-init.gentoo';
+        } else {
+            $init_file = 'init-scripts/psad-init.generic';
+        }
     }
 
     if ($init_dir and &is_root()) {
         &logr("[+] Copying $init_file -> ${init_dir}/$init_name\n");
         copy $init_file, "${init_dir}/$init_name" or die "[*] Could not copy ",
             "$init_file -> ${init_dir}/$init_name: $!";
-        &perms_ownership("${init_dir}/$init_name", 0744);
+        if ($is_systemd) {
+            &perms_ownership("${init_dir}/$init_name", 0644);
+        } else {
+            &perms_ownership("${init_dir}/$init_name", 0744);
+        }
         &enable_psad_at_boot($distro);
         $installed_init_script = 1;
     }
@@ -777,7 +805,11 @@ sub install() {
     }
     if ($installed_init_script) {
         if ($init_dir) {
-            &logr("\n[+] To start psad, run \"${init_dir}/psad start\"\n");
+            if ($is_systemd) {
+                &logr("\n[+] To start psad, run \"$cmds{'systemctl'} start psad\"\n");
+            } else {
+                &logr("\n[+] To start psad, run \"${init_dir}/psad start\"\n");
+            }
         } else {
             &logr("\n[+] To start psad, run ${USRSBIN_DIR}/psad\"\n");
         }
@@ -904,9 +936,9 @@ sub uninstall() {
         if (kill 0, $pid) {
             print "[+] Stopping psad daemons!\n";
             if (-e "${init_dir}/psad") {  ### prefer this for old versions
-                system "${init_dir}/psad stop";
+                &run_cmd("${init_dir}/psad stop");
             } else {
-                system "${USRSBIN_DIR}/psad --Kill";
+                &run_cmd("${USRSBIN_DIR}/psad --Kill");
             }
         }
     }
@@ -929,8 +961,8 @@ sub uninstall() {
             warn "[*] Could not remove ${USRSBIN_DIR}/kmsgsd!!!\n";
     }
     if (-e "${init_dir}/psad") {
-        print "[+] Removing ${init_dir}/psad\n";
-        unlink "${init_dir}/psad";
+        print "[+] Removing ${init_dir}/$init_name\n";
+        unlink "${init_dir}/$init_name";
     }
     if (-d $config{'PSAD_CONF_DIR'}) {
         print "[+] Removing configuration directory: $config{'PSAD_CONF_DIR'}";
@@ -965,10 +997,6 @@ sub uninstall() {
             rmtree $dir;
         }
     }
-    print "[+] You will need to remove psad-specific config lines ",
-        "from your syslog config.\n",
-        "    Your original syslog config at psad install time is preserved here:\n",
-        "    $config{'CONF_ARCHIVE_DIR'}\n";
     print "\n[+] psad has been uninstalled!\n";
     return;
 }
@@ -1002,16 +1030,20 @@ sub stop_psad() {
         chomp $pid;
         if (kill 0, $pid) {
             print "[+] Stopping running psad daemons.\n";
-            if (-x "$init_dir/psad") {
-                system "$init_dir/psad stop";
+            if (-x "$init_dir/$init_name") {
+                if ($is_systemd) {
+                    &run_cmd("$cmds{'systemctl'} stop psad");
+                } else {
+                    &run_cmd("$init_dir/$init_name stop");
+                }
                 ### if psad is still running then use -K
                 if (kill 0, $pid) {
-                    system "$USRSBIN_DIR/psad -K";
+                    &run_cmd("$USRSBIN_DIR/psad -K");
                 }
             } else {
                 ### psad may have been started from the command line
                 ### without using the init script, so stop with -K
-                system "$USRSBIN_DIR/psad -K";
+                &run_cmd("$USRSBIN_DIR/psad -K");
             }
         }
     }
@@ -1082,13 +1114,13 @@ sub install_perl_module() {
         }
 
         $ENV{'PERL5LIB'} = $config{'PSAD_LIBS_DIR'};  ### for module dependencies
-        system "$cmds{'make'} clean"
+        &run_cmd("$cmds{'make'} clean")
             if -e 'Makefile' or -e 'makefile' or -e 'GNUmakefile';
-        system "$cmds{'perl'} Makefile.PL PREFIX=$config{'PSAD_LIBS_DIR'} " .
-            "LIB=$config{'PSAD_LIBS_DIR'}";
-        system "$cmds{'make'}";
-#        system "$cmds{'make'} test";
-        system "$cmds{'make'} install";
+        &run_cmd("$cmds{'perl'} Makefile.PL PREFIX=$config{'PSAD_LIBS_DIR'} " .
+            "LIB=$config{'PSAD_LIBS_DIR'}");
+        &run_cmd("$cmds{'make'}");
+#        &run_cmd("$cmds{'make'} test");
+        &run_cmd("$cmds{'make'} install");
         chdir $src_dir or die "[*] Could not chdir $src_dir: $!";
 
         print "\n\n";
@@ -1097,6 +1129,31 @@ sub install_perl_module() {
     return;
 }
 
+sub run_cmd_get_output() {
+    my ($cmd_path, $args) = @_;
+    my $cmd = $cmd_path;
+    $cmd .= " $args" if $args;
+    open CMD, "$cmd |" or die "[*] Could not ",
+        "execute $cmd: $!";
+    my @lines = <CMD>;
+    close CMD;
+    return \@lines;
+}
+
+sub look_for_process() {
+    my $process_re = shift;
+    for (@{&run_cmd_get_output($cmds{'ps'}, 'auxww')}) {
+        return 1 if /$process_re/;
+    }
+    return 0;
+}
+
+sub run_cmd() {
+    my $cmd = shift;
+    &logr("[+] CMD: '$cmd'\n");
+    return (system $cmd) >> 8;
+}
+
 sub set_home_net() {
     my $file = shift;
 
@@ -1228,7 +1285,7 @@ sub download_signatures() {
         &logr("[-] The $wgetCmd var is not a valid path for wget, " .
             "skipping sig install.\n");
     }
-    system "$wgetCmd $config{'SIG_UPDATE_URL'}";
+    &run_cmd("$wgetCmd $config{'SIG_UPDATE_URL'}");
 
     unless (-e 'signatures') {
         &logr("[-] Could not download signatures, continuing with install.\n");
@@ -1579,9 +1636,7 @@ sub test_syslog_config() {
     $test_port++ while defined $used_ports{$test_port};
 
     ### make sure the interface is actually up
-    my $uprv = (system "$cmds{'ifconfig'} lo up") >> 8;
-
-    if ($uprv) {
+    unless (&run_cmd("$cmds{'ifconfig'} lo up")) {
         &logr("[-] Could not bring up the loopback interface.\n" .
             "    Hoping the syslog reconfig will work anyway.\n");
         return 0;
@@ -1632,7 +1687,7 @@ sub test_syslog_config() {
     if ($start_kmsgsd) {
         ### briefly start kmsgsd just long enough to test syslog
         ### with a packet to port 5000 (or higher).
-        unless (((system "${USRSBIN_DIR}/kmsgsd")>>8) == 0) {
+        unless (&run_cmd("${USRSBIN_DIR}/kmsgsd") == 0) {
             &logr("[-] Could not start kmsgsd to test syslog.\n" .
                 "    Send email to Michael Rash (mbr\@cipherdyne.org)\n");
             return 0;
@@ -1641,8 +1696,8 @@ sub test_syslog_config() {
 
     ### insert a rule to deny traffic to the loopback
     ### interface on $test_port
-    system "$cmds{'iptables'} -I INPUT 1 -i lo -p tcp --dport " .
-        "$test_port -j LOG --log-prefix \"test_DROP \"";
+    &run_cmd("$cmds{'iptables'} -I INPUT 1 -i lo -p tcp --dport " .
+        "$test_port -j LOG --log-prefix \"test_DROP \"");
 
     open FWDATA, "$config{'FW_DATA_FILE'}" or
         die "[*] Could not open $config{'FW_DATA_FILE'}: $!";
@@ -1673,7 +1728,7 @@ sub test_syslog_config() {
     }
 
     ### remove the testing firewall rule
-    system "$cmds{'iptables'} -D INPUT 1";
+    &run_cmd("$cmds{'iptables'} -D INPUT 1");
 
     ### remove the any new test_DROP lines we just created
     ### (this probably is not necessary because psad is not
@@ -2035,7 +2090,7 @@ sub archive() {
     ### move $file into the archive directory
     copy $file, "${base}1" or die "[*] Could not copy ",
         "$file -> ${base}1: $!";
-    system "$cmds{'gzip'} ${base}1";
+    &run_cmd("$cmds{'gzip'} ${base}1");
     chdir $curr_pwd or die $!;
     $archived_old = 1;
     return;
@@ -2046,14 +2101,20 @@ sub enable_psad_at_boot() {
 
     return unless &is_root();
 
-    if (&query_yes_no("[+] Enable psad at boot time ([y]/n)?  ",
+    unless (&query_yes_no("[+] Enable psad at boot time ([y]/n)?  ",
                 $ACCEPT_YES_DEFAULT)) {
+        return;
+    }
+
+    if ($is_systemd) {
+        &run_cmd("$cmds{'systemctl'} enable psad");
+    } else {
         if ($distro eq 'redhat' or $distro eq 'fedora') {
-            system "$cmds{'chkconfig'} --add $init_name";
+            &run_cmd("$cmds{'chkconfig'} --add $init_name");
         } elsif ($distro eq 'gentoo') {
-            system "$cmds{'rc-update'} add $init_name default";
+            &run_cmd("$cmds{'rc-update'} add $init_name default");
         } elsif ($distro eq 'ubuntu') {
-            system "$cmds{'update-rc.d'} $init_name defaults";
+            &run_cmd("$cmds{'update-rc.d'} $init_name defaults");
         } else {
 
             ### get the current run level
@@ -2186,13 +2247,13 @@ sub install_manpage() {
     &full_mkdir($mpath, 0755);
     my $mfile = "${mpath}/${manpage}";
     &logr("[+] Installing $manpage man page at $mfile\n");
-    copy $manpage, $mfile or die "[*] Could not copy $manpage to ",
+    copy "doc/$manpage", $mfile or die "[*] Could not copy doc/$manpage to ",
         "$mfile: $!";
     &perms_ownership($mfile, 0644);
     &logr("[+] Compressing manpage $mfile\n");
     ### remove the old one so gzip doesn't prompt us
     unlink "${mfile}.gz" if -e "${mfile}.gz";
-    system "$cmds{'gzip'} $mfile";
+    &run_cmd("$cmds{'gzip'} $mfile");
     return;
 }
 
@@ -2332,8 +2393,12 @@ Usage: install.pl [options]
                                    derived.
     --init-dir <path>            - Specify path to the init directory (the
                                    default is $init_dir).
+    --systemd-init-dir <path>    - Specify path to the systemd init directory
+                                   (the default is $systemd_init_dir).
     --init-name <name>           - Specify the name for the psad init
                                    script (the default is $init_name).
+    --systemd-init-name <name>   - Specify the name for the systemd psad init
+                                   file (the default is $systemd_init_name).
     --install-syslog-fifo        - Add the installation of the psadfifo
                                    (this is not usually necessary since
                                    the default is to enable
