@@ -7,21 +7,29 @@ use Getopt::Long 'GetOptions';
 use strict;
 
 #==================== config =====================
-my $logfile        = 'test.log';
-my $output_dir     = 'output';
-my $conf_dir       = 'conf';
-my $run_dir        = 'run';
-my $scans_dir      = 'scans';
-my $test_install_dir = 'psad-install';
+my $logfile        = './test.log';
+my $output_dir     = './output';
+my $conf_dir       = './conf';
+my $run_dir        = './run';
+my $scans_dir      = './scans';
+my $test_install_dir = './psad-install';
 my $syn_scan_file  = 'syn_scan_1000_1500';
+my $syn_sweep_file = 'syn_sweep';
+my $syn_scan_multi_dst_file  = 'syn_scan_1000_1500_multi_dst';
 my $topera_syn_scan_file = 'topera_ipv6_syn_scan_no_ip_opts';
 my $topera_syn_scan_with_opts_file = 'topera_ipv6_syn_scan_with_ip_opts';
+my $mirai_scan_file = 'mirai_default_credentials_scan';
+my $syn_scan_localhost_file = 'syn_scan_1000_1500_localhost';
+my $syn_scan_zeroip_file = 'syn_scan_1000_1500_zeroip';
 my $fin_scan_file  = 'fin_scan_1000_1150';
 my $xmas_scan_file = 'xmas_scan_1000_1150';
 my $null_scan_file = 'null_scan_1000_1150';
 my $ack_scan_file  = 'ack_scan_1000_1150';
 my $udp_scan_file  = 'udp_scan_1000_1150';
 my $proto_scan_file = 'proto_scan';
+my $igmp_traffic_file = 'ipv4_igmp';
+my $fwknop_pkt_file = 'fwknop_spa_pkt';
+my $syslog_time_fmt1 = 'syslog_time_fmt1.log';
 my $ms_sql_server_sig_match_file  = 'ms_sql_server_sig_match';
 my $ipv6_ms_sql_server_sig_match_file  = 'ipv6_ms_sql_server_sig_match';
 my $no_ms_sql_server_sig_match_file = "$conf_dir/signatures_no_ms_sql_server_sig";
@@ -45,6 +53,7 @@ my $cmd_out_tmp    = 'cmd.out';
 my $default_conf   = "$conf_dir/default_psad.conf";
 my $ignore_udp_conf = "$conf_dir/ignore_udp.conf";
 my $ignore_tcp_conf = "$conf_dir/ignore_tcp.conf";
+my $ignore_igmp_conf = "$conf_dir/ignore_igmp.conf";
 my $ignore_intf_conf = "$conf_dir/ignore_intf.conf";
 my $auto_blocking_conf = "$conf_dir/auto_blocking.conf";
 my $auto_dl5_blocking_conf = "$conf_dir/auto_min_dl5_blocking.conf";
@@ -97,10 +106,10 @@ my @args_cp = @ARGV;
 
 exit 1 unless GetOptions(
     'psad-path=s'         => \$psadCmd,
-    'test-include=s'      => \$test_include,
-    'include=s'           => \$test_include,  ### synonym
-    'test-exclude=s'      => \$test_exclude,
-    'exclude=s'           => \$test_exclude,  ### synonym
+    'test-include=s'      => \@tests_to_include,
+    'include=s'           => \@tests_to_include,  ### synonym
+    'test-exclude=s'      => \@tests_to_exclude,
+    'exclude=s'           => \@tests_to_exclude,  ### synonym
     'test-system-install' => \$test_system_install,
     'enable-auto-block-tests' => \$enable_auto_block_tests,
     'List-mode'           => \$list_mode,
@@ -146,6 +155,15 @@ my @tests = (
         'exec_err' => $NO,
         'fatal'    => $NO
     },
+    {
+        'category' => 'operations',
+        'detail'   => '--Version',
+        'err_msg'  => '--Version mismatch',
+        'function' => \&version_check,
+        'cmdline'  => "$psadCmd -V -c $default_conf $normal_root_override_str",
+        'exec_err' => $NO,
+        'fatal'    => $NO
+    },
     {
         'category' => 'operations',
         'detail'   => 'config dump+validate',
@@ -182,7 +200,7 @@ my @tests = (
         'category'  => 'operations',
         'detail'    => '--fw-analyze',
         'err_msg'   => 'could not analyze fw policy',
-        'positive_output_matches' => [qr/Parsing.*iptables/, qr/Parsing.*ip6tables/],
+        'positive_output_matches' => [qr/Parsing.*rules/, qr/Parsing.*rules/],
         'match_all' => $MATCH_ALL_RE,
         'function'  => \&generic_exec,
         'cmdline'   => "$psadCmd --test-mode --fw-analyze -c $default_conf $normal_root_override_str",
@@ -195,7 +213,7 @@ my @tests = (
         'err_msg'   => 'could not get psad status',
         'function'  => \&generic_exec,
         'cmdline'   => "$psadCmd --test-mode -S -c $default_conf $normal_root_override_str",
-        'exec_err'  => $NO,
+#        'exec_err'  => $NO,
         'fatal'     => $NO
     },
     {
@@ -204,7 +222,7 @@ my @tests = (
         'err_msg'   => 'could not get psad status summary',
         'function'  => \&generic_exec,
         'cmdline'   => "$psadCmd --test-mode -S --status-summary -c $default_conf $normal_root_override_str",
-        'exec_err'  => $NO,
+#        'exec_err'  => $NO,
         'fatal'     => $NO
     },
     {
@@ -245,6 +263,37 @@ my @tests = (
         'exec_err'  => $NO,
         'fatal'     => $NO
     },
+    {
+        'category'  => 'operations',
+        'detail'    => 'IPv4 SYN scan multi-dst detection',
+        'err_msg'   => 'did not detect SYN scan',
+        'positive_output_matches' => [qr/Top\s\d+\sattackers/i,
+                qr/scanned\sports.*?1000\-1500\b/i,
+                qr/Source\sOS/i, qr/BACKDOOR/i,
+                qr/IP\sstatus/i,
+                qr/Total\sscan\sdestinations\:\s4/i,
+                qr/192\.168\.10\.55/],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data -m $scans_dir/" .
+                &fw_type() . "/$syn_scan_multi_dst_file -c $default_conf $normal_root_override_str",
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
+    {
+        'category'  => 'operations',
+        'detail'    => 'IPv4 SYN sweep detection',
+        'err_msg'   => 'did not detect SYN scan',
+        'positive_output_matches' => [qr/Top\s\d+\sattackers/i,
+                qr/port sweep.*port\s1083\b/i],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data -m $scans_dir/" .
+                &fw_type() . "/$syn_sweep_file -c $default_conf $normal_root_override_str",
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
+
     {
         'category'  => 'operations',
         'detail'    => 'IPv6 Topera SYN scan detection',
@@ -278,7 +327,19 @@ my @tests = (
         'exec_err'  => $NO,
         'fatal'     => $NO
     },
-
+    {
+        'category'  => 'operations',
+        'detail'    => 'IPv4 detect Mirai default credentials scans',
+        'err_msg'   => 'did not detect Mirai scan',
+        'positive_output_matches' => [qr/Top\s\d+\sattackers/i,
+                qr/Mirai\sbotnet\sdefault\scredentials\sscan/i],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data -m $scans_dir/" .
+                &fw_type() . "/$mirai_scan_file -c $default_conf $normal_root_override_str",
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
     {
         'category'  => 'operations',
         'detail'    => 'IPv4 MS SQL Server communication attempt detection',
@@ -402,6 +463,34 @@ my @tests = (
         'exec_err'  => $NO,
         'fatal'     => $NO
     },
+    {
+        'category'  => 'operations',
+        'detail'    => 'IPv4 fwknop SPA packet detection',
+        'err_msg'   => 'did not detect fwknop SPA packet',
+        'positive_output_matches' => [qr/Top\s\d+\sattackers/i,
+                qr/fwknop\sSingle\sPacket/i,
+                qr/IP\sstatus/i,
+                qr/192\.168\.10\.55/],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data -m $scans_dir/" .
+                &fw_type() . "/$fwknop_pkt_file -c $default_conf $normal_root_override_str",
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
+    {
+        'category'  => 'operations',
+        'detail'    => 'syslog time format (1)',
+        'err_msg'   => 'did not parse syslog time format',
+        'positive_output_matches' => [qr/syslog\shostname\:\sservername/,
+                qr/timestamp\:\s2015\-03\-08T/],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data -m $scans_dir/" .
+                &fw_type() . "/$syslog_time_fmt1 -c $default_conf $normal_root_override_str",
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
 
     {
         'category'  => 'operations',
@@ -514,6 +603,37 @@ my @tests = (
         'exec_err'  => $NO,
         'fatal'     => $NO
     },
+    {
+        'category'  => 'operations',
+        'detail'    => 'IPv4 <BLOCK> skipping 127.0.0.1',
+        'err_msg'   => 'blocked localhost',
+        'positive_output_matches' => [qr/Skipping\sIP.*from\sauto/i,
+         ],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data --auto-dl $dl5_ipv4_auto_dl_file " .
+                "-m $scans_dir/" .  &fw_type() . "/$syn_scan_localhost_file -c $auto_dl5_blocking_conf " .
+                "$normal_root_override_str --analysis-auto-block",
+        'auto_block_test' => $YES,
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
+    {
+        'category'  => 'operations',
+        'detail'    => 'IPv4 <BLOCK> skipping 0.0.0.0',
+        'err_msg'   => 'blocked localhost',
+        'positive_output_matches' => [qr/Skipping\sIP.*from\sauto/i,
+         ],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data --auto-dl $dl5_ipv4_auto_dl_file " .
+                "-m $scans_dir/" .  &fw_type() . "/$syn_scan_zeroip_file -c $auto_dl5_blocking_conf " .
+                "$normal_root_override_str --analysis-auto-block",
+        'auto_block_test' => $YES,
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
+
     {
         'category'  => 'operations',
         'detail'    => 'IPv4 <BLOCK> SYN scan',
@@ -772,6 +892,18 @@ my @tests = (
         'exec_err'  => $NO,
         'fatal'     => $NO
     },
+    {
+        'category'  => 'operations',
+        'detail'    => 'psad.conf ignore IPv4 IGMP traffic',
+        'err_msg'   => 'did not ignore TCP traffic',
+        'negative_output_matches' => [qr/SRC\:\s+192\.168\.10\.55/],
+        'match_all' => $MATCH_ALL_RE,
+        'function'  => \&generic_exec,
+        'cmdline'   => "$psadCmd --test-mode -A --analysis-write-data --auto-dl $dl5_ipv4_auto_dl_file " .  ### psad.conf IGNORE_PROTOCOLS trumps auto_dl
+                "-m $scans_dir/" .  &fw_type() . "/$igmp_traffic_file -c $ignore_igmp_conf $normal_root_override_str",
+        'exec_err'  => $NO,
+        'fatal'     => $NO
+    },
     {
         'category'  => 'operations',
         'detail'    => 'psad.conf ignore eth1 traffic',
@@ -1113,6 +1245,45 @@ sub install_test_dir() {
     return $rv;
 }
 
+sub version_check() {
+    my $test_hr = shift;
+
+    my $version_file = '../VERSION';
+
+    my $version = '';
+    if (-e $version_file) {
+        open V, "< $version_file" or die $!;
+        $version = <V>;
+        close $version;
+        chomp $version;
+    } else {
+        die "[*] version file: $version_file does not exist";
+    }
+
+    my $rv = &generic_exec($test_hr);
+
+    if ($rv) {
+        my $output_version = '';
+        open C, "< $current_test_file" or die $!;
+        while (<C>) {
+            if (/psad\sv(\S+)/) {
+                $output_version = $1;
+                last;
+            }
+        }
+        close C;
+        if ($output_version) {
+            unless ($version eq $output_version) {
+                $rv = 0;
+            }
+        } else {
+            $rv = 0;
+        }
+    }
+
+    return $rv;
+}
+
 sub generic_exec() {
     my $test_hr = shift;
 
@@ -1304,13 +1475,6 @@ sub init() {
         unlink $logfile or die $!;
     }
 
-    if ($test_include) {
-        @tests_to_include = split /\s*,\s*/, $test_include;
-    }
-    if ($test_exclude) {
-        @tests_to_exclude = split /\s*,\s*/, $test_exclude;
-    }
-
     push @tests_to_exclude, 'BLOCK' unless $enable_auto_block_tests;
 
     return;
@@ -1374,5 +1538,17 @@ sub logr() {
 }
 
 sub usage() {
-    ### FIXME
+    print <<_HELP_;
+
+$0 [options]
+
+    -i, --include <test>        - Include tests that match <str>.
+    --exclude <test>            - Exclude tests that match <str>.
+    --enable-auto-block-tests   - Run auto blocking tests.
+    -t, --test-limit <num>      - Run <num> tests.
+    -L, --List-mode             - List available tests.
+    -h, --help                  - Print help and exit.
+
+_HELP_
+    exit 0;
 }
